package be.iminds.ilabt.jfed.restapi.dao;

import be.iminds.ilabt.jfed.fedmon.util.Clock;
import be.iminds.ilabt.jfed.restapi.JFedRestApiApplication;
import be.iminds.ilabt.jfed.restapi.JFedRestApiConfiguration;
import be.iminds.ilabt.jfed.restapi.TestDB;
import be.iminds.ilabt.jfed.restapi.json.SpeaksForCredential;
import io.dropwizard.jdbi.DBIFactory;
import io.dropwizard.setup.Environment;
import io.dropwizard.testing.junit.DropwizardAppRule;
import org.junit.*;
import org.skife.jdbi.v2.DBI;

import java.sql.SQLException;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;

import static org.mockito.Mockito.mock;

public class SpeaksForCredentialDoaTest {
    @ClassRule
    public static final DropwizardAppRule<JFedRestApiConfiguration> RULE =
            new DropwizardAppRule<JFedRestApiConfiguration>(JFedRestApiApplication.class, "../jfed-rest-api-TESTING.yml");

    private static TestDB testDB;
    private static DBI jdbi;
    private static SpeaksForCredentialDoa speaksForCredentialDoa;
    //lock to make sure that nothing tries to parallellise these tests
    private static Lock lock = new ReentrantLock();
    @BeforeClass
    public static void setUp() throws Exception {
        Clock.set(new Clock.DefaultTestTimeProvider());

        testDB = new TestDB(RULE);
        testDB.syncTableStructure();
        testDB.refillDB();

        JFedRestApiConfiguration configuration = RULE.getConfiguration();
        Environment environment = RULE.getEnvironment();
        final DBIFactory factory = new DBIFactory();
        jdbi = factory.build(environment, configuration.getDataSourceFactory(), "postgresql");
        speaksForCredentialDoa = jdbi.onDemand(SpeaksForCredentialDoa.class);
        assert speaksForCredentialDoa != null;
    }
    @AfterClass
    public static void tearDown() throws Exception {
        assert testDB != null;
        jdbi.close(speaksForCredentialDoa);
        jdbi = null;
        speaksForCredentialDoa = null;
        testDB = null;
        System.gc();
        Clock.reset();
    }

    private SpeaksForCredentialTestCollection speaksForCredentialTestCollection;

    @Before
    public void beforeEachMethod() throws SQLException {
        lock.lock();

        assert testDB != null;
        testDB.refillDB();

        assert speaksForCredentialDoa != null;

        speaksForCredentialTestCollection = new SpeaksForCredentialTestCollection();
    }


    @After
    public void afterEachMethod() throws SQLException {
        lock.unlock();
    }

    @Test
    public void testInsertAndFind0() throws Exception {
        assert speaksForCredentialDoa != null;
        SpeaksForCredential expected = speaksForCredentialTestCollection.getByIndex(0);
        speaksForCredentialDoa.upsert(expected);
        SpeaksForCredential actual= speaksForCredentialDoa.find(expected.getSpeaker(), expected.getSpeakingFor());
        System.out.println("testFind0 res = "+actual);
        assertThat(actual, is(notNullValue()));
        speaksForCredentialTestCollection.assertSame(actual, expected);
    }

    @Test
    public void testInsertAndFind1() throws Exception {
        assert speaksForCredentialDoa != null;
        SpeaksForCredential expected = speaksForCredentialTestCollection.getByIndex(1);
        speaksForCredentialDoa.upsert(expected);
        SpeaksForCredential actual= speaksForCredentialDoa.find(expected.getSpeaker(), expected.getSpeakingFor());
        System.out.println("testFind1 res = "+actual);
        assertThat(actual, is(notNullValue()));
        speaksForCredentialTestCollection.assertSame(actual, expected);
    }

    @Test
    public void testInsertAndFind2() throws Exception {
        assert speaksForCredentialDoa != null;
        SpeaksForCredential expected = speaksForCredentialTestCollection.getByIndex(2);
        speaksForCredentialDoa.upsert(expected);
        SpeaksForCredential actual= speaksForCredentialDoa.find(expected.getSpeaker(), expected.getSpeakingFor());
        System.out.println("testFind2 res = "+actual);
        assertThat(actual, is(notNullValue()));
        speaksForCredentialTestCollection.assertSame(actual, expected);
    }

    @Test
    public void testInsertAndFind3() throws Exception {
        assert speaksForCredentialDoa != null;
        SpeaksForCredential expected = speaksForCredentialTestCollection.getByIndex(3);
        speaksForCredentialDoa.upsert(expected);
        SpeaksForCredential actual= speaksForCredentialDoa.find(expected.getSpeaker(), expected.getSpeakingFor());
        System.out.println("testFind3 res = "+actual);
        assertThat(actual, is(notNullValue()));
        speaksForCredentialTestCollection.assertSame(actual, expected);
    }

    @Test
    public void testInsertAndUpsert1() throws Exception {
        assert speaksForCredentialDoa != null;
        SpeaksForCredential expectedFirst = speaksForCredentialTestCollection.getByIndex(0);
        SpeaksForCredential expectedAfter = speaksForCredentialTestCollection.getByIndex(2);

        speaksForCredentialDoa.upsert(expectedFirst);
        SpeaksForCredential actualFirst = speaksForCredentialDoa.find(expectedFirst.getSpeaker(), expectedFirst.getSpeakingFor());
        assertThat(actualFirst, is(notNullValue()));
        speaksForCredentialTestCollection.assertSame(actualFirst, expectedFirst);

        //0 has later expire, so 2 should overwrite it

        speaksForCredentialDoa.upsert(expectedAfter);
        SpeaksForCredential actualAfter = speaksForCredentialDoa.find(expectedAfter.getSpeaker(), expectedAfter.getSpeakingFor());
        assertThat(actualAfter, is(notNullValue()));
        speaksForCredentialTestCollection.assertSame(actualAfter, expectedAfter);
    }

    @Test
    public void testInsertAndUpsert2() throws Exception {
        assert speaksForCredentialDoa != null;
        SpeaksForCredential expectedFirst = speaksForCredentialTestCollection.getByIndex(2);
        SpeaksForCredential secondUpsert = speaksForCredentialTestCollection.getByIndex(0);
        SpeaksForCredential expectedAfter = expectedFirst;

        speaksForCredentialDoa.upsert(expectedFirst);
        SpeaksForCredential actualFirst = speaksForCredentialDoa.find(expectedFirst.getSpeaker(), expectedFirst.getSpeakingFor());
        assertThat(actualFirst, is(notNullValue()));
        speaksForCredentialTestCollection.assertSame(actualFirst, expectedFirst);

        //2 has later expire, so 0 should NOT overwrite it

        speaksForCredentialDoa.upsert(secondUpsert);
        SpeaksForCredential actualAfter = speaksForCredentialDoa.find(secondUpsert.getSpeaker(), secondUpsert.getSpeakingFor());
        assertThat(actualAfter, is(notNullValue()));
        speaksForCredentialTestCollection.assertSame(actualAfter, expectedAfter);
    }

    @Test
    public void testInsertAndUpsertOther1() throws Exception {
        assert speaksForCredentialDoa != null;
        SpeaksForCredential expectedFirst = speaksForCredentialTestCollection.getByIndex(0);

        speaksForCredentialDoa.upsert(expectedFirst);
        SpeaksForCredential actualFirst = speaksForCredentialDoa.find(expectedFirst.getSpeaker(), expectedFirst.getSpeakingFor());
        assertThat(actualFirst, is(notNullValue()));
        speaksForCredentialTestCollection.assertSame(actualFirst, expectedFirst);

        speaksForCredentialDoa.upsert(speaksForCredentialTestCollection.getByIndex(1));
        speaksForCredentialDoa.upsert(speaksForCredentialTestCollection.getByIndex(3));
        SpeaksForCredential actualAfter = speaksForCredentialDoa.find(expectedFirst.getSpeaker(), expectedFirst.getSpeakingFor());
        assertThat(actualAfter, is(notNullValue()));
        speaksForCredentialTestCollection.assertSame(actualAfter, expectedFirst);
    }
}
